package cn.wise.sc.cement.business.service;

import cn.wise.sc.cement.business.entity.NormProduction;
import cn.wise.sc.cement.business.entity.NormProductionStatistics;
import cn.wise.sc.cement.business.model.BaseResponse;
import cn.wise.sc.cement.business.model.PageQuery;
import cn.wise.sc.cement.business.model.vo.NormProductionVo;
import cn.wise.sc.cement.business.model.vo.ProductionVo;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;

import java.util.List;

/**
 * <p>
 * 服务类
 * </p>
 *
 * @author ztw
 * @since 2020-09-23
 */
public interface INormProductionService extends IService<NormProduction> {

    /**
     * 创建新的标准产值配置
     *
     * @param normProduction 标准产值
     * @return BaseResponse
     */
    BaseResponse<NormProduction> addObj(NormProduction normProduction);

    /**
     * 分页查询
     *
     * @param pageQuery 分页条件
     * @return 数据
     */
    BaseResponse<IPage<NormProductionVo>> getPage(PageQuery pageQuery);

    /**
     * 编辑标准产值
     *
     * @param production 标准产值
     * @return bool
     */
    BaseResponse<Boolean> edit(NormProduction production);

    /**
     * 启用禁用标准产值
     *
     * @param id id
     * @return bool
     */
    BaseResponse<Integer> activeOrForbidden(Integer id);

    /**
     * 标准产值统计
     *
     * @param start 开始时间
     * @param end   结束时间
     * @return BaseResponse
     */
    BaseResponse<List<NormProductionStatistics>> normProductionStatistics(Long start, Long end, String name);

    /**
     * 标准产值统计 单人详情
     *
     * @param userId 用户id
     * @param start  开始时间
     * @param end    结束时间
     * @return 详细信息
     */
    List<NormProduction.NormProductionDetail> normProductionDetails(Integer userId,
                                                                    Long start, Long end);

    List<ProductionVo> production(String name,Long start,Long end);
}
